﻿using SqlSugar;
using System;
using System.Runtime.Serialization;

namespace WCS.Entity
{
    /// <summary>
    /// AGV任务中间表
    /// </summary>
    [SugarTable(nameof(WCS_AGVTask), "AGV任务中间表")]
    [DataContract]
    public class WCS_AGVTask
    {
        /// <summary>
        /// ID
        /// </summary>
        [SugarColumn(ColumnDescription = "ID"), DataMember(Order = 0)]
        public int ID { get; set; }

        /// <summary>
        /// AGVID
        /// </summary>
        [SugarColumn(ColumnDescription = "AGVID", Length = 20), DataMember(Order = 1)]
        public string AGVID { get; set; }

        /// <summary>
        /// 任务类型
        /// </summary>
        [SugarColumn(ColumnDescription = "任务类型")]
        [DataMember(Order = 2)]
        public AGVTaskType TaskType { get; set; }

        /// <summary>
        /// WCS AGV任务状态
        /// </summary>
        [SugarColumn(ColumnDescription = "WCS AGV任务状态")]
        [DataMember(Order = 3)]
        public AGVTaskStatus Status { get; set; }

        /// <summary>
        /// AGV自身状态
        /// </summary>
        [SugarColumn(ColumnDescription = "AGV自身状态")]
        [DataMember(Order = 4)]
        public AGVTaskStatus AGVStatus { get; set; }

        /// <summary>
        /// 车间
        /// </summary>
        [SugarColumn(ColumnDescription = "车间")]
        [DataMember(Order = 5)]
        public int Workshop { get; set; }

        /// <summary>
        /// 站台
        /// </summary>
        [SugarColumn(ColumnDescription = "站台", Length = 20)]
        [DataMember(Order = 6)]
        public string Station { get; set; }

        /// <summary>
        /// 位置
        /// </summary>
        [SugarColumn(ColumnDescription = "位置", Length = 20)]
        [DataMember(Order = 7)]
        public string Position { get; set; }

        /// <summary>
        /// 货物数量(1.两个位置放1个货物，2.两个位置放2个货物）
        /// </summary>
        [SugarColumn(ColumnDescription = "货物数量(1.两个位置放1个货物，2.两个位置放2个货物）")]
        [DataMember(Order = 11)]
        public int Goodsnum { get; set; }

        /// <summary>
        /// 创建时间
        /// </summary>
        [SugarColumn(ColumnDescription = "创建时间")]
        [DataMember(Order = 8)]
        public DateTime CreateTime { get; set; }

        /// <summary>
        /// WCS更新时间
        /// </summary>
        [SugarColumn(ColumnDescription = "WCS更新时间")]
        [DataMember(Order = 9)]
        public DateTime? UpdateTime { get; set; }

        /// <summary>
        /// AGV更新时间
        /// </summary>
        [SugarColumn(ColumnDescription = "AGV更新时间")]
        [DataMember(Order = 10)]
        public DateTime? AGVUpdateTime { get; set; }

        /// <summary>
        /// 更新用户
        /// 记录用户ID
        /// </summary>
        [SugarColumn(ColumnDescription = "更新用户")]
        [DataMember(Order = 11)]
        public string UPDATEUSER { get; set; }
    }

    /// <summary>
    /// AGV任务状态
    /// </summary>
    public enum AGVTaskStatus
    {
        /// <summary>
        /// 新建
        /// </summary>
        NewBuild = 0,

        /// <summary>
        /// 确认
        /// </summary>
        Confirm = 5,

        /// <summary>
        /// 执行
        /// </summary>
        Execution = 10,

        /// <summary>
        /// 请求/允许 AGV请求，WCS允许 根据任务类型判断取放货
        /// 工位一
        /// </summary>
        RequestOrPermission1 = 20,

        /// <summary>
        /// 取放完成
        /// 工位一
        /// </summary>
        Complete1 = 21,

        /// <summary>
        /// 请求/允许 AGV请求，WCS允许 根据任务类型判断取放货
        /// 工位二
        /// </summary>
        RequestOrPermission2 = 30,

        /// <summary>
        /// 取放完成
        /// 工位二
        /// </summary>
        Complete2 = 31,

        /// <summary>
        /// 任务完成
        /// </summary>
        MissionCompleted = 99,

        /// <summary>
        /// 完成扫码
        /// </summary>
        CompleteScanCode = 100,

        /// <summary>
        ///  取消
        /// </summary>
        Cancel = 106
    }

    /// <summary>
    /// AGV任务类型
    /// </summary>
    public enum AGVTaskType
    {
        /// <summary>
        /// 入库
        /// </summary>
        EnterDepot = 1,

        /// <summary>
        /// 叫料
        /// </summary>
        CallForMaterial = 2
    }
}
