using Newtonsoft.Json;
using ServiceCenter;
using ServiceCenter.Redis;
using ServiceCenter.SqlSugars;
using ServiceCenter.Virtual_PLC;
using SqlSugar;
using System.Text;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol;
using WCS.Entity.Protocol.BCR;
using WCS.Entity.Protocol.Station;
using WCS.Service.Systems;

namespace WCS.Service
{
    /// <summary>
    /// 
    /// </summary>
    public class Worker : BackgroundService
    {
        /// <summary>
        /// ¼
        /// </summary>
        private readonly ILogger<Worker> _logger;

        /// <summary>
        /// 캯
        /// </summary>
        /// <param name="logger">¼</param>
        public Worker(ILogger<Worker> logger)
        {
            _logger = logger;
        }

        public static readonly string WcsDlog = "WCSDlog";
        public static readonly string Wcsdb = "WCSDB";

        /// <summary>
        ///  ִ
        /// </summary>
        /// <param name="stoppingToken">ֹͣ</param>
        /// <returns></returns>
        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            if (stoppingToken.IsCancellationRequested)
                return;

            #region ־

            //var logConfigText = RedisHub.Default.Check("LogConfigText") ?? throw new Exception("Redislog4net");
            //var logConfig = JsonConvert.DeserializeObject<LogConfig>(logConfigText);
            //LogHub.SetConfigInfo(logConfig!);

            #endregion ־

            _logger.LogInformation("WCSʼ");
            Configs.ProtocolProxyBaseType = typeof(ProtocolProxy);
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            Configs.StringEncoding = Encoding.UTF8;
            var warehouseName = RedisHub.Default.Check("WarehouseName") ?? throw new Exception("Redisòֿ");
            if (string.IsNullOrEmpty(warehouseName)) throw new Exception("Redisòֿ");
            ServiceHub.SetWarehouseName(warehouseName);

            #region ʼݿ

            var dbConnectionStrings = RedisHub.Default.Check("DbConnectionStrings") ?? throw new Exception("Redisݿ");
            ServiceHub.DbConnectionStrings = JsonConvert.DeserializeObject<List<DataBaseConnectionString>>(dbConnectionStrings);
            if (ServiceHub.DbConnectionStrings != null)
            {
                if (ServiceHub.DbConnectionStrings.All(v => v.Key != Wcsdb)) throw new Exception("DbConnectionStringsWCSݿַ");
                if (ServiceHub.DbConnectionStrings.All(v => v.Key == Wcsdb && !v.IsDefault)) throw new Exception("DbConnectionStringsWCSݿΪĬݿ");
                if (ServiceHub.DbConnectionStrings.All(v => v.Key != WcsDlog)) throw new Exception("DbConnectionStringsWCS־ݿַ");
            }

            //Ϣ
            List<ConnectionConfig> connectionConfigs = new List<ConnectionConfig>();
            foreach (var connectionString in ServiceHub.DbConnectionStrings!)
            {
                connectionConfigs.Add(new ConnectionConfig()
                {
                    ConfigId = connectionString.Key,
                    ConnectionString = connectionString.ConnectionString,//ӷִ
                    DbType = connectionString.DbType,//ݿ
                    IsAutoCloseConnection = true,//trueҪֶclose
                });
            };
            SqlSugarHelper.SetDb(new SqlSugarScope(connectionConfigs));
            //SqlSugarHelper.SetDb(new SqlSugarScope(connectionConfigs, db =>
            //{
            //    db.GetConnectionScope(Wcsdb).Aop.OnLogExecuting = (sql, pars) =>
            //    {
            //        Console.WriteLine(db.GetConnectionScope(Wcsdb).Ado.Connection.ConnectionString + "\r\n " + sql);
            //        Console.WriteLine();
            //        Console.WriteLine();
            //    };
            //    db.GetConnectionScope(WcsDlog).Aop.OnLogExecuting = (sql, pars) =>
            //    {
            //        Console.WriteLine(db.GetConnectionScope(WcsDlog).Ado.Connection.ConnectionString + "\r\n " + sql);
            //        Console.WriteLine();
            //        Console.WriteLine();
            //    };
            //}));

            //ʼݿ
            SqlSugarHelper.Do(db =>
            {
                foreach (var connectionString in ServiceHub.DbConnectionStrings!)
                {
                    var _db = db.Connect.GetConnectionScope(connectionString.Key);
                    switch (connectionString.Key)
                    {
                        case "WCSDB"://WCSݿ
                            SqlSugarHelper.SetDefault(connectionString.Key);
                            _db.CodeFirst.InitTables(typeof(WCS_PLC));
                            _db.CodeFirst.InitTables(typeof(WCS_DATABLOCK));
                            _db.CodeFirst.InitTables(typeof(WCS_DEVICEHdr));
                            _db.CodeFirst.InitTables(typeof(WCS_PathHdr));
                            _db.CodeFirst.InitTables(typeof(WCS_PathDtl));
                            _db.CodeFirst.InitTables(typeof(WCS_ROUTE));
                            _db.CodeFirst.InitTables(typeof(WCS_TASK));
                            _db.CodeFirst.InitTables(typeof(WCS_TASK_DTL));
                            _db.CodeFirst.InitTables(typeof(WCS_TASK_OLD));
                            _db.CodeFirst.InitTables(typeof(WCS_PlcData));
                            _db.CodeFirst.InitTables(typeof(WCS_AGVTask));
                            _db.CodeFirst.InitTables(typeof(WCS_DEVICEPROTOCOL));
                            _db.CodeFirst.InitTables(typeof(WCS_GROUPMEMBER));

                            break;

                        case "WCSDlog"://WCS־ݿ
                            SqlSugarHelper.SetDlog(connectionString.Key);
                            _db.DbMaintenance.CreateDatabase();
                            _db.CodeFirst.InitTables(typeof(WCS_BCR80));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_RGV520));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_RGV521));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_RGV523));
                            //_db.CodeFirst.InitTables(typeof(WCS_SRM520));
                            //_db.CodeFirst.InitTables(typeof(WCS_SRM521));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_SRM537));
                            _db.CodeFirst.InitTables(typeof(WCS_Station520));
                            _db.CodeFirst.InitTables(typeof(WCS_Station521));
                            _db.CodeFirst.InitTables(typeof(WCS_Station523));
                            _db.CodeFirst.InitTables(typeof(WCS_Station91));

                            break;

                        default: //
                            break;
                    };
                };
            });

            #endregion ʼݿ

            #region PLC

            var isOpenVirtualPlc = RedisHub.Default.Check("isOpenVirtualPLC") ?? throw new Exception("RedsiǷPLC");
            if (isOpenVirtualPlc == "1")
            {
                var plcDataConnectionString = RedisHub.Default.Check("plcDataConnectionString") ?? throw new Exception("RedsiPLCʹõRedisַ");

                //нṹҪĽṹ
                var list = new List<PLCData>();
                SqlSugarHelper.Do(db =>
                {
                    var _db = db.Connect;
                    var dataBlocks = _db.Queryable<WCS_DATABLOCK>().Includes(v => v.PLC).ToList();
                    list.AddRange(dataBlocks.Select(dataBlock => new PLCData()
                    {
                        IP = dataBlock.PLC.IP,
                        DB = dataBlock.NO,
                        Length = dataBlock.LENGTH,
                        DataLength = dataBlock.DATALENGTH,
                    }));
                });
                PlcData.Init(plcDataConnectionString).InitPlcData(list);

                ServiceHub.AddSystemMode(SystemMode.plc);
            }

            #endregion PLC

            var a = typeof(IStation520);

            //־¼s
            Configs.PublishEvent += () =>
            {
                //WMS.UploadDevInfo();
                //ProtocolProxy.Do();
            };
            //쳣
            Configs.UploadException = (d, s) =>
            {
                //if (s == "ӿڵ") return;
                //if (ProtocolProxy.AllDatas.ContainsKey(d))
                //{
                //    ProtocolProxy.AllDatas[d].Info = s;
                //    //ProtocolProxy.AllDatas[d].Frame = LogicHandler.Frame;
                //}
                //WMS.TaskException(d, s);
            };
            //PLC
            Configs.PLCAccessorCreater = new PLCAccessors.PLCAccessorsCreater();
            try
            {
                SqlSugarHelper.Do(db =>
                {
                    var _db = db.Connect;
                    //ȡDBдЭ
                    var dbProtocols = _db.Queryable<WCS_DEVICEPROTOCOL>().Includes(v => v.DB, p => p.PLC).ToList();
                    foreach (var dbProtocol in dbProtocols)
                    {
#pragma warning disable CS8604 // ͲΪ null
                        Add(Type.GetType(dbProtocol.DB.PROTOCOL), dbProtocol.DEVICECODE, dbProtocol.POSITION, dbProtocol.DB, dbProtocol.DB.PLC);
#pragma warning restore CS8604 // ͲΪ null
                    }
                });

                #region е

                World.StartAll();

                #endregion е

                _logger.LogInformation("WCSɹ");

                #region ݲɼ

                while (true)
                {
                    Ltc.GetSystem<DataCollectionSysyem>().Invoke(true);
                }

                #endregion ݲɼ
            }
            catch (Exception ex)
            {
                _logger.LogError("WCSʧ{0}", ex.Message);
            }
        }

        /// <summary>
        ///  Э
        /// </summary>
        /// <param name="type">Э</param>
        /// <param name="code">豸</param>
        /// <param name="position">ַ</param>
        /// <param name="db">db</param>
        /// <param name="plc">PLC</param>
        public static void Add(Type type, string code, int position, WCS_DATABLOCK db, WCS_PLC plc)
        {
            var info = new ProtocolInfo
            {
                Position = position,
                DBInfo = new DBInfo
                {
                    No = (ushort)db.NO,
                    PLCInfo = new PLCInfo
                    {
                        IP = plc.IP,
                        Port = plc.PORT,
                        Rack = plc.RACK,
                        Slot = plc.SLOT,
                        Type = Core.PLCType.Siemens
                    }
                }
            };
            try
            {
                Protocols.Add(type, code, info);
            }
            catch (Exception ex)
            {
                var a = ex;
            }
        }
    }
}
